package com.ajax;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *
 * @author tgiunipero
 */
public class ComposerDBAO {

    // connection instance
    private Connection con = null;

    // returns a list of composer matches
    public List getComposers(String userInput) throws SQLException {

        // list to hold all composer matches
        List<Composer> composerMatches = null;

        // Composer object for retaining data
        Composer composer;

        // create SQL query
        String query = "SELECT * FROM composer";

        try {
            // connect to database
            DataSource dataSource = getJdbcConnectionPool();
            con = dataSource.getConnection();

            // prepare query
            PreparedStatement ps = con.prepareStatement(query);

            // resultset to hold data from query
            ResultSet rs = ps.executeQuery();

            composerMatches = new ArrayList();

            // loop through rows from the generated result set
            while (rs.next()) {
                // declare an instance of Composer to match
                // returned data with object properties
                composer = new Composer();

                // set the data to properties
                composer.setId(rs.getString("id"));
                composer.setFirstName(rs.getString("first_name"));
                composer.setLastName(rs.getString("last_name"));

                if ( // user input matches first name
                     composer.getFirstName().toLowerCase().startsWith(userInput) ||
                     // user input matches last name
                     composer.getLastName().toLowerCase().startsWith(userInput) ||
                     // user input matches full name
                     composer.getFirstName().toLowerCase().concat(" ").concat(composer.getLastName().toLowerCase()).startsWith(userInput)) {

                    composerMatches.add(composer);
                }
            }

        } catch (Exception e) {
            System.out.println(e.getMessage());

        } finally {
            // close the connection so it can be returned to
            // the connection pool then return the list
            con.close();
            return composerMatches;
        }
    }


    // returns a composer object whose id equals id
    public Composer getComposerDetails(String id) throws SQLException {

        // create SQL query
        String query = "SELECT * FROM composer WHERE id = ?";

        // Composer object to hold retrieved data
        Composer composer = null;

        try {
            // connect to database
            DataSource dataSource = getJdbcConnectionPool();
            con = dataSource.getConnection();

            // prepare query
            PreparedStatement ps = con.prepareStatement(query);
            ps.setString(1, id);

            // set up result set to retain all data
            ResultSet rs = ps.executeQuery();

            if (rs.next()) {
                rs.getRow();

                // declare an instance of Composer to match
                // returned data with object properties
                composer = new Composer();

                // set the data to properties
                composer.setId(rs.getString("id"));
                composer.setFirstName(rs.getString("first_name"));
                composer.setLastName(rs.getString("last_name"));
                composer.setPeriod(rs.getString("period"));
            }

        } catch (Exception e) {
            System.out.println(e.getMessage());

        } finally {
            // close the connection so it can be returned to
            // the connection pool then return the list
            con.close();
            return composer;
        }
    }

    private DataSource getJdbcConnectionPool() throws NamingException {
        Context c = new InitialContext();
        return (DataSource) c.lookup("java:comp/env/jdbc/composers");
    }
}