package com.ajax;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author tgiunipero
 */
public class ComposerServlet extends HttpServlet {

    private ServletContext context;

    @Override
    public void init(ServletConfig config) throws ServletException {
        this.context = config.getServletContext();
    }

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        PrintWriter out;

        // declare variables for incoming parameters
        String userInput = request.getParameter("input");
        String userAction = request.getParameter("action");

        // check if userInput parameter exists
        if (userInput != null) {
            userInput = userInput.trim().toLowerCase();

            // check if user sent empty string
            if (!userInput.equals("")) {

                // specify XML content for HttpServletResponse
                response.setContentType("text/xml");
                response.setHeader("Cache-Control", "no-cache");

                // prepare and write output to HttpServletResponse
                out = response.getWriter();

                try {
                    ComposerDBAO dbao = new ComposerDBAO();

                    List<Composer> composers = dbao.getComposers(userInput);

                    if (!composers.isEmpty()) {

                        out.write("<composers>");

                        for (Composer c : composers) {

                            Composer composer = c;

                            out.write("<composer>" +
                                      "<id>" + composer.getId() + "</id>" +
                                      "<firstName>" + composer.getFirstName() + "</firstName>" +
                                      "<lastName>" + composer.getLastName() + "</lastName>" +
                                      "</composer>");
                        }
                        out.write("</composers>");

                    } else {
                        //no matching content
                        response.setStatus(HttpServletResponse.SC_NO_CONTENT);
                    }

                } catch (SQLException ex) {
                    Logger.getLogger(ComposerServlet.class.getName()).log(Level.SEVERE, null, ex);
                } finally {
                    out.close();
                }
            } else {
                //nothing to show
                response.setStatus(HttpServletResponse.SC_NO_CONTENT);
            }
            
        // check if action parameter exists
        } else if (userAction != null) {

            if (userAction.equals("lookup")) {

                String id = request.getParameter("id");

                if ((id != null)) {

                    ComposerDBAO dbao = new ComposerDBAO();
                    try {
                        // get target composer from data store
                        Composer c = dbao.getComposerDetails(id);

                        // put the target composer in the request scope
                        request.setAttribute("composer", c);
                    } catch (SQLException ex) {
                        Logger.getLogger(ComposerServlet.class.getName()).log(Level.SEVERE, null, ex);
                    }

                    context.getRequestDispatcher("/composer.jsp").forward(request, response);
                }
            }
        } else {
            // if neither userInput nor action parameter exists, flag error
            context.getRequestDispatcher("/error.jsp").forward(request, response);
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}