package com.ajax;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author tgiunipero
 */
public class ComposerServlet extends HttpServlet {

    private ServletContext context;

    @Override
    public void init(ServletConfig config) throws ServletException {
        this.context = config.getServletContext();
    }

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        PrintWriter out;

        ComposerData compData = new ComposerData();
        HashMap composers = compData.getComposers();

        // declare variables for incoming parameters
        String userInput = request.getParameter("input");
        String userAction = request.getParameter("action");

        // check if userInput parameter exists
        if (userInput != null) {
            userInput = userInput.trim().toLowerCase();

            // check if user sent empty string
            if (!userInput.equals("")) {

                // specify XML content for HttpServletResponse
                response.setContentType("text/xml");
                response.setHeader("Cache-Control", "no-cache");

                // prepare and write output to HttpServletResponse
                out = response.getWriter();

                StringBuffer sb = new StringBuffer();
                Iterator iter = composers.keySet().iterator();

                while (iter.hasNext()) {
                    String thisKey = (String) iter.next();
                    Composer composer = (Composer) composers.get(thisKey);

                    if ( // user input matches first name
                         composer.getFirstName().toLowerCase().startsWith(userInput) ||
                         // user input matches last name
                         composer.getLastName().toLowerCase().startsWith(userInput) ||
                         // user input matches full name
                         composer.getFirstName().toLowerCase().concat(" ").concat(composer.getLastName().toLowerCase()).startsWith(userInput)) {

                        sb.append("<composer>");
                        sb.append("<id>" + composer.getId() + "</id>");
                        sb.append("<firstName>" + composer.getFirstName() + "</firstName>");
                        sb.append("<lastName>" + composer.getLastName() + "</lastName>");
                        sb.append("</composer>");
                    }
                }

                // if composer matches were found, enclose all
                //  matches inside <composers> tags and write output
                if (sb.length() > 0) {
                    out.write("<composers>" + sb.toString() + "</composers>");
                } else {
                    //no matching content
                    response.setStatus(HttpServletResponse.SC_NO_CONTENT);
                }

                out.close();

            } else {
                //nothing to show
                response.setStatus(HttpServletResponse.SC_NO_CONTENT);
            }

        // check if action parameter exists
        } else if (userAction != null) {

            if (userAction.equals("lookup")) {

                String id = request.getParameter("id");

                // put the target composer in the request scope to display
                if ((id != null) && composers.containsKey(id.trim())) {
                    request.setAttribute("composer", composers.get(id));
                    context.getRequestDispatcher("/composer.jsp").forward(request, response);
                }
            }
        } else {
            // if neither userInput nor action parameter exists, flag error
            context.getRequestDispatcher("/error.jsp").forward(request, response);
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}