package dojo.indians;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashSet;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author tgiunipero
 */
public class TribeDataManager {

    private static List<Tribe> indians = new ArrayList();

    // Initialize List of Indians
    static {

        indians.add(new Tribe("Eskimo-Aleut", "Arctic", "Alaska Natives"));

        indians.add(new Tribe("Northern Athabaskan", "Subarctic", "Alaska Natives"));

        indians.add(new Tribe("Yok-Utian", "Californian tribes", "Western United States"));
        indians.add(new Tribe("Pacific Coast Athabaskan", "Californian tribes", "Western United States"));
        indians.add(new Tribe("Coast Miwok", "Californian tribes", "Western United States"));
        indians.add(new Tribe("Yurok", "Californian tribes", "Western United States"));
        indians.add(new Tribe("Palaihnihan", "Californian tribes", "Western United States"));

        indians.add(new Tribe("Interior Salish", "Plateau tribes", "Western United States"));
        indians.add(new Tribe("Plateau Penutian", "Plateau tribes", "Western United States"));

        indians.add(new Tribe("Uto-Aztecan", "Great Basin tribes", "Western United States"));

        indians.add(new Tribe("Pacific Coast Athabaskan", "Pacific Northwest Coast", "Western United States"));
        indians.add(new Tribe("Coast Salish", "Pacific Northwest Coast", "Western United States"));

        indians.add(new Tribe("Uto-Aztecan", "Southwestern tribes", "Western United States"));
        indians.add(new Tribe("Yuman", "Southwestern tribes", "Western United States"));
        indians.add(new Tribe("Southern Athabaskan", "Southwestern tribes", "Western United States"));

        indians.add(new Tribe("Siouan", "Plains Indians", "Central United States"));
        indians.add(new Tribe("Plains Algonquian", "Plains Indians", "Central United States"));
        indians.add(new Tribe("Southern Athabaskan", "Plains Indians", "Central United States"));

        indians.add(new Tribe("Iroquoian", "Northeastern Woodlands tribes", "Eastern United States"));
        indians.add(new Tribe("Central Algonquian", "Northeastern Woodlands tribes", "Eastern United States"));
        indians.add(new Tribe("Eastern Algonquian", "Northeastern Woodlands tribes", "Eastern United States"));

        indians.add(new Tribe("Muskogean", "Southeastern tribes", "Eastern United States"));
        indians.add(new Tribe("Siouan", "Southeastern tribes", "Eastern United States"));
        indians.add(new Tribe("Catawban", "Southeastern tribes", "Eastern United States"));
        indians.add(new Tribe("Iroquoian", "Southeastern tribes", "Eastern United States"));

    }

    /**
     * Returns all data contained in the 'indians' ArrayList as JSON-structured
     * contents required by Dojo's <code>ItemFileReadStore</code>
     * <br><br>
     * Example:
     *
     *  <pre>{
     *      "identifier": "name",
     *      "items": [
     *          {
     *              "children": [
     *                  {"_reference": "Subarctic"},
     *                  {"_reference": "Arctic"}
     *              ],
     *              "name": "Alaska Natives",
     *              "type": "region"
     *          },
     *          {
     *              "children": [{"_reference": "Northern Athabaskan"}],
     *              "name": "Subarctic",
     *              "type": "category"
     *          },
     *          {
     *              "children": [{"_reference": "Eskimo-Aleut"}],
     *              "name": "Arctic",
     *              "type": "category"
     *          },
     *          {
     *              "name": "Eskimo-Aleut",
     *              "type": "tribe"
     *          },
     *          {
     *              "name": "Northern Athabaskan",
     *              "type": "tribe"
     *          }
     *      ],
     *      "label": "name"
     *  }</pre>
     */
    public static JSONObject getIndiansAsJSONObject() throws JSONException {

        JSONObject jo = new JSONObject();
        JSONArray itemsArray = new JSONArray();

        jo.put("identifier", "name");
        jo.put("label", "name");

        // add regions
        addRegionsToJSONArray(itemsArray);

        // add categories
        addCategoriesToJSONArray(itemsArray);

        // add tribes
        addTribesToJSONArray(itemsArray);

        jo.put("items", itemsArray);
        return jo;
    }

    /**
     * Returns a <code>HashSet</code> containing all elements within the 'indians'
     * <code>ArrayList</code> that match a <code>Tribe</code> property
     * <br><br>
     * Expects "names", "categories", or "regions" as a parameter
     */
    private static HashSet getAll(String properties) {

        HashSet elements = new HashSet();

        for (Tribe t : indians) {
            Tribe tribe = t;

            if (properties.equals("names")) {
                String name = tribe.getName();
                elements.add(name);
            } else if (properties.equals("categories")) {
                String category = tribe.getCategory();
                elements.add(category);
            } else if (properties.equals("regions")) {
                String region = tribe.getRegion();
                elements.add(region);
            }
        }
        return elements;
    }

    // helper method for addRegionsToJSONArray(JSONArray itemsArray)
    private static HashSet getCategoriesByRegion(String region) {

        HashSet categories = new HashSet();

        for (Tribe t : indians) {
            Tribe tribe = t;
            String r = tribe.getRegion();

            if (r.equals(region)) {
                String c = tribe.getCategory();
                categories.add(c);
            }
        }
        return categories;
    }

    // helper method for addCategoriesToJSONArray(JSONArray itemsArray)
    private static HashSet getTribesByCategory(String category) {

        HashSet tribes = new HashSet();

        for (Tribe t : indians) {
            Tribe tribe = t;
            String c = tribe.getCategory();

            if (c.equals(category)) {
                String n = tribe.getName();
                tribes.add(n);
            }
        }
        return tribes;
    }

    // helper method for getIndiansAsJSONObject()
    private static JSONArray addRegionsToJSONArray(JSONArray itemsArray) throws JSONException {

        JSONObject regionObject = null;

        HashSet regions = getAll("regions");
        Iterator iter = regions.iterator();

        // get regions
        while (iter.hasNext()) {

            regionObject = new JSONObject();

            String thisRegion = iter.next().toString();
            regionObject.put("name", thisRegion);
            regionObject.put("type", "region");

            // get categories within each region
            JSONArray categoriesArray = new JSONArray();

            HashSet categories = getCategoriesByRegion(thisRegion);

            if (!categories.isEmpty()) {

                Iterator categoryIterator = categories.iterator();
                while (categoryIterator.hasNext()) {

                    String thisCategory = categoryIterator.next().toString();
                    JSONObject tribeObject = new JSONObject();
                    tribeObject.put("_reference", thisCategory);

                    categoriesArray.put(tribeObject);
                }

                regionObject.put("children", categoriesArray);
            }

            itemsArray.put(regionObject);
        }

        return itemsArray;
    }

    // helper method for getIndiansAsJSONObject()
    private static JSONArray addCategoriesToJSONArray(JSONArray itemsArray) throws JSONException {

        JSONObject categoryObject = null;

        HashSet categories = getAll("categories");
        Iterator iter = categories.iterator();

        // get categories
        while (iter.hasNext()) {

            categoryObject = new JSONObject();

            String thisCategory = iter.next().toString();
            categoryObject.put("name", thisCategory);
            categoryObject.put("type", "category");

            // get tribes within each category
            JSONArray tribesArray = new JSONArray();

            HashSet tribes = getTribesByCategory(thisCategory);

            if (!tribes.isEmpty()) {
                Iterator tribeIterator = tribes.iterator();
                while (tribeIterator.hasNext()) {

                    String thisTribe = tribeIterator.next().toString();
                    JSONObject tribeObject = new JSONObject();
                    tribeObject.put("_reference", thisTribe);

                    tribesArray.put(tribeObject);
                }

                categoryObject.put("children", tribesArray);
            }

            itemsArray.put(categoryObject);
        }

        return itemsArray;
    }

    // helper method for getIndiansAsJSONObject()
    private static JSONArray addTribesToJSONArray(JSONArray itemsArray) throws JSONException {

        JSONObject tribeObject = null;

        for (Tribe tribe : indians) {

            String thisName = tribe.getName();
            Boolean duplicate = false;

            //check if name not already included
            for (Tribe t : indians) {

                if (!tribe.equals(t)) {

                    for (int i = 0; i < itemsArray.length(); i++) {
                        JSONObject jo = itemsArray.getJSONObject(i);
                        if (jo.optString("type").equals("tribe")) {
                            if (jo.optString("name").equals(thisName)) {
                                duplicate = true;
                            }
                        }
                    }
                }
            }

            if (duplicate != true) {
                tribeObject = new JSONObject();
                tribeObject.put("name", thisName);
                tribeObject.put("type", "tribe");

                itemsArray.put(tribeObject);
            }

        }

        return itemsArray;
    }
}